"""
SMS backend for sending text messages using Twilio.
"""
from typing import List, Optional

from django.conf import settings  # type: ignore
from django.core.exceptions import ImproperlyConfigured  # type: ignore

from sms.backends.base import BaseSmsBackend
from sms.message import Message

try:
    from twilio.rest import Client  # type: ignore
    HAS_TWILIO = True
except ImportError:
    HAS_TWILIO = False


class SmsBackend(BaseSmsBackend):
    def __init__(self, fail_silently: bool = False, **kwargs) -> None:
        super().__init__(fail_silently=fail_silently, **kwargs)

        if not HAS_TWILIO and not self.fail_silently:
            raise ImproperlyConfigured(
                "You're using the SMS backend "
                "'sms.backends.twilio.SmsBackend' without having "
                "'twilio' installed. Install 'twilio' or use "
                "another SMS backend."
            )

        account_sid: Optional[str] = getattr(settings, 'TWILIO_ACCOUNT_SID')
        if not account_sid and not self.fail_silently:
            raise ImproperlyConfigured(
                "You're using the SMS backend "
                "'sms.backends.twilio.SmsBackend' without having the "
                "setting 'TWILIO_ACCOUNT_SID' set."
            )

        auth_token: Optional[str] = getattr(settings, 'TWILIO_AUTH_TOKEN')
        if not auth_token and not self.fail_silently:
            raise ImproperlyConfigured(
                "You're using the SMS backend "
                "'sms.backends.twilio.SmsBackend' without having the "
                "setting 'TWILIO_AUTH_TOKEN' set."
            )

        self.client = None
        if HAS_TWILIO:
            self.client = Client(account_sid, auth_token)

    def send_messages(self, messages: List[Message]) -> int:
        if not self.client:
            return 0

        msg_count: int = 0
        for message in messages:
            for recipient in message.recipients:
                try:
                    self.client.messages.create(
                        to=recipient,
                        from_=message.originator,
                        body=message.body
                    )
                except Exception as exc:
                    if not self.fail_silently:
                        raise exc
                msg_count += 1
        return msg_count



project = 'django-sms'
copyright = '2023, Roald Nefs'
author = 'Roald Nefs'
release = '0.7.0'

extensions = ['m2r2']
templates_path = ['_templates']
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']
source_suffix = ['.rst', '.md']

html_theme = 'alabaster'
html_theme_options = {
    "description": "A Django app for sending SMS with interchangeable backends.",
    "show_powered_by": False,
    "github_user": "roaldnefs",
    "github_repo": "django-sms",
    "github_button": True,
    "github_banner": True,
    "show_related": False,
}
html_static_path = ['_static']



"""
SMS backend for usage in a test environment.
"""
from typing import List

import sms
from sms.backends.base import BaseSmsBackend
from sms.message import Message


class SmsBackend(BaseSmsBackend):
    """
    An SMS backend for use during test session.

    The test connection stores text messages in a dummy outbox, rather than
    sending them out on the wire.

    The dummy outbox is accessible through the outbox instance attribute.
    """
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        if not hasattr(sms, 'outbox'):
            sms.outbox = []  # type: ignore

    def send_messages(self, messages: List[Message]) -> int:
        """Redirect messages to the dummy outbox."""
        msg_count: int = 0
        for message in messages:
            sms.outbox.append(message)  # type: ignore
            msg_count += 1
        return msg_count



"""
SMS backend that writes text messages to console instead of sending them.
"""
import sys
import threading

from typing import List

from sms.backends.base import BaseSmsBackend
from sms.message import Message


class SmsBackend(BaseSmsBackend):
    def __init__(self, *args, **kwargs) -> None:
        self.stream = kwargs.pop('stream', sys.stdout)
        self._lock = threading.RLock()
        super().__init__(*args, **kwargs)

    def write_message(self, message: Message) -> int:
        msg_count = 0
        for recipient in message.recipients:
            msg_data = (
                f"from: {message.originator}\n"
                f"to: {recipient}\n"
                f"{message.body}"
            )
            self.stream.write(f'{msg_data}\n')
            self.stream.write('-' * 79)
            self.stream.write('\n')
            msg_count += 1
        return msg_count

    def send_messages(self, messages: List[Message]) -> int:
        """Write all text messages to the stream in a thread-safe way."""
        msg_count: int = 0
        if not messages:
            return msg_count
        with self._lock:
            try:
                stream_created = self.open()
                for message in messages:
                    count = self.write_message(message)
                    self.stream.flush()  # flush after each message
                    msg_count += count
                if stream_created:
                    self.close()
            except Exception:
                if not self.fail_silently:
                    raise
        return msg_count



import os
import sys
import shutil
import tempfile

from typing import List, Type, Optional
from io import StringIO

from unittest.mock import MagicMock

from django.dispatch import receiver  # type: ignore
from django.test import SimpleTestCase, override_settings  # type: ignore

import sms
from sms import send_sms
from sms.backends import dummy, locmem, filebased
from sms.backends.base import BaseSmsBackend
from sms.message import Message
from sms.signals import post_send
from sms.utils import message_from_bytes, message_from_binary_file


class BaseSmsBackendTests:
    sms_backend: Optional[str] = None

    def setUp(self) -> None:
        self.settings_override = override_settings(
            SMS_BACKEND=self.sms_backend
        )
        self.settings_override.enable()

    def tearDown(self) -> None:
        self.settings_override.disable()


class SmsTests(SimpleTestCase):

    def test_dummy_backend(self) -> None:
        """
        Make sure that dummy backends return correct number of sent text
        messages.
        """
        connection = dummy.SmsBackend()
        message = Message()
        self.assertEqual(
            connection.send_messages([message, message, message]),
            3
        )

    def test_backend_arg(self) -> None:
        """Test backend argument of sms.get_connection()."""
        self.assertIsInstance(
            sms.get_connection('sms.backends.dummy.SmsBackend'),
            dummy.SmsBackend
        )
        with tempfile.TemporaryDirectory() as tmp_dir:
            self.assertIsInstance(
                sms.get_connection(
                    'sms.backends.filebased.SmsBackend',
                    file_path=tmp_dir
                ),
                filebased.SmsBackend
            )
        if sys.platform == 'win32':
            msg = ('_getfullpathname: path should be string, bytes or '
                   'os.PathLike, not object')
        else:
            msg = 'expected str, bytes or os.PathLike object, not object'
        with self.assertRaisesMessage(TypeError, msg):
            sms.get_connection(
                'sms.backends.filebased.SmsBackend',
                file_path=object()
            )
        self.assertIsInstance(sms.get_connection(), locmem.SmsBackend)

    def test_custom_backend(self) -> None:
        """Test cutoms backend defined in this suite."""
        connection = sms.get_connection('tests.custombackend.SmsBackend')
        self.assertTrue(hasattr(connection, 'test_outbox'))
        message = Message('Content', '0600000000', ['0600000000'])
        connection.send_messages([message])  # type: ignore
        self.assertEqual(len(connection.test_outbox), 1)  # type: ignore

    @override_settings(SMS_BACKEND='sms.backends.locmem.SmsBackend')
    def test_send_sms(self) -> None:
        """
        Test send_sms with the recipients specified as a string to remain
        compatibility with django-sms<=0.0.4.
        """
        send_sms('Content', '0600000000', '0600000000')
        self.assertEqual(len(sms.outbox), 1)  # type: ignore
        self.assertIsInstance(sms.outbox[0].recipients, list)  # type: ignore


class LocmemBackendTests(BaseSmsBackendTests, SimpleTestCase):
    sms_backend: str = 'sms.backends.locmem.SmsBackend'

    def flush_mailbox(self) -> None:
        sms.outbox = []  # type: ignore

    def tearDown(self) -> None:
        super().tearDown()
        self.flush_mailbox()

    def test_locmem_shared_messages(self) -> None:
        """
        Make sure that the locmem backend populates the outbox.
        """
        connections: List[Type[BaseSmsBackend]] = [
            locmem.SmsBackend(),  # type: ignore
            locmem.SmsBackend()  # type: ignore
        ]
        message = Message('Content', '0600000000', ['0600000000'])
        for connection in connections:
            connection.send_messages([message])  # type: ignore
        self.assertEqual(len(sms.outbox), 2)  # type: ignore


class ConsoleBackendTests(BaseSmsBackendTests, SimpleTestCase):
    sms_backend: str = 'sms.backends.console.SmsBackend'

    def test_console_stream_kwarg(self) -> None:
        """
        The console backend can be pointed at an arbitrary stream.
        """
        stream = StringIO()
        connection = sms.get_connection(
            'sms.backends.console.SmsBackend',
            stream=stream
        )
        message = Message('Content', '0600000000', ['0600000000'])
        connection.send_messages([message])  # type: ignore
        messages = stream.getvalue().split('\n' + ('-' * 79) + '\n')
        self.assertIn('from: ', messages[0])


class FileBasedBackendTests(BaseSmsBackendTests, SimpleTestCase):
    sms_backend = 'sms.backends.filebased.SmsBackend'

    def setUp(self) -> None:
        super().setUp()
        self.tmp_dir = self.mkdtemp()
        self.addCleanup(shutil.rmtree, self.tmp_dir)
        self._settings_override = override_settings(SMS_FILE_PATH=self.tmp_dir)
        self._settings_override.enable()

    def tearDown(self) -> None:
        self._settings_override.disable()
        super().tearDown()

    def mkdtemp(self) -> str:
        return tempfile.mkdtemp()

    def flush_mailbox(self) -> None:
        for filename in os.listdir(self.tmp_dir):
            os.unlink(os.path.join(self.tmp_dir, filename))

    def get_mailbox_content(self) -> List[Message]:
        messages: List[Message] = []
        for filename in os.listdir(self.tmp_dir):
            with open(os.path.join(self.tmp_dir, filename), 'rb') as fp:
                session = fp.read().split(b'\n' + (b'-' * 79) + b'\n')
            messages.extend(message_from_bytes(m) for m in session if m)
        return messages

    def test_file_sessions(self) -> None:
        """Make sure opening a connection creates a new file"""
        message = Message(
            'Here is the message',
            '+12065550100',
            ['+441134960000']
        )
        connection = sms.get_connection()
        connection.send_messages([message])  # type: ignore

        self.assertEqual(len(os.listdir(self.tmp_dir)), 1)
        tmp_file = os.path.join(self.tmp_dir, os.listdir(self.tmp_dir)[0])
        with open(tmp_file, 'rb') as fp:
            message = message_from_binary_file(fp)
        self.assertEqual(message.originator, '+12065550100')
        self.assertEqual(message.recipients, ['+441134960000'])


class MessageBirdBackendTests(BaseSmsBackendTests, SimpleTestCase):
    sms_backend = 'sms.backends.messagebird.SmsBackend'

    def setUp(self) -> None:
        super().setUp()
        self._settings_override = override_settings(
            MESSAGEBIRD_ACCESS_KEY='fake_access_key'
        )
        self._settings_override.enable()

    def tearDown(self) -> None:
        self._settings_override.disable()
        super().tearDown()

    def test_send_messages(self) -> None:
        """Test send_messages with the MessageBird backend."""
        message = Message(
            'Here is the message',
            '+12065550100',
            ['+441134960000']
        )

        connection = sms.get_connection()
        connection.client.message_create = MagicMock()  # type: ignore
        connection.send_messages([message])  # type: ignore
        connection.client.message_create.assert_called_with(  # type: ignore
            '+12065550100',
            ['+441134960000'],
            'Here is the message'
        )


class TwilioBackendTests(BaseSmsBackendTests, SimpleTestCase):
    sms_backend = 'sms.backends.twilio.SmsBackend'

    def setUp(self) -> None:
        super().setUp()
        self._settings_override = override_settings(
            TWILIO_ACCOUNT_SID='fake_account_sid',
            TWILIO_AUTH_TOKEN='fake_auth_token',
        )
        self._settings_override.enable()

    def tearDown(self) -> None:
        self._settings_override.disable()
        super().tearDown()

    def test_send_messages(self) -> None:
        """Test send_messages with the Twilio backend."""
        message = Message(
            'Here is the message',
            '+12065550100',
            ['+441134960000']
        )

        connection = sms.get_connection()
        connection.client.messages.create = MagicMock()  # type: ignore
        connection.send_messages([message])  # type: ignore
        connection.client.messages.create.assert_called_with(  # type: ignore
            to='+441134960000',
            from_='+12065550100',
            body='Here is the message'
        )


class SignalTests(SimpleTestCase):

    def flush_mailbox(self) -> None:
        sms.outbox = []  # type: ignore

    def tearDown(self) -> None:
        super().tearDown()
        self.flush_mailbox()

    def test_receiver_post_send_signal(self) -> None:
        """Make sure the post_send signal is called."""
        @receiver(post_send)
        def f(instance, **kwargs):
            self.body = instance.body
            self.state = True
        self.state = False
        self.body = None

        body = 'Here is the message'
        message = Message(body, '+12065550100', ['+441134960000'])
        message.send()

        self.assertTrue(self.state)
        self.assertEqual(body, self.body)



#!/usr/bin/env python

import sys
import django  # type: ignore

from django.test.runner import DiscoverRunner  # type: ignore
from django.conf import settings  # type: ignore


settings.configure(
    DATABASES={'default': {'ENGINE': 'django.db.backends.sqlite3'}},
    SECRET_KEY="it's a secret to everyone",
    SMS_BACKEND='sms.backends.locmem.SmsBackend',
)


def main() -> None:
    django.setup()
    runner = DiscoverRunner(failfast=True, verbosity=1)
    failures = runner.run_tests([], interactive=True)
    sys.exit(failures)


if __name__ == '__main__':
    main()



"""
Dummy SMS backend that does nothing.
"""
from typing import List

from sms.backends.base import BaseSmsBackend
from sms.message import Message


class SmsBackend(BaseSmsBackend):
    def send_messages(self, messages: List[Message]) -> int:
        return len(list(messages))



from typing import Type, Optional, List, TYPE_CHECKING

from django.conf import settings  # type: ignore

from sms.signals import post_send

if TYPE_CHECKING:
    from sms.backends.base import BaseSmsBackend


class Message:
    """A container for text message information."""
    def __init__(
        self,
        body: str = '',
        originator: Optional[str] = None,
        recipients: Optional[List[str]] = None,
        connection: Optional[Type['BaseSmsBackend']] = None
    ) -> None:
        """
        Initialize a single text message (which can be sent to multiple
        recipients).
        """
        if recipients:
            if isinstance(recipients, str):
                raise TypeError(
                    '"recipients" argument must be a list or tuple'
                )
            self.recipients = recipients
        else:
            self.recipients = []
        self.originator = originator or getattr(
            settings, 'DEFAULT_FROM_SMS', ''
        )
        self.body = body or ''
        self.connection = connection

    def get_connection(
        self,
        fail_silently: bool = False
    ) -> Type['BaseSmsBackend']:
        from sms import get_connection
        if not self.connection:
            self.connection = get_connection(fail_silently=fail_silently)
        return self.connection

    def send(self, fail_silently: bool = False) -> int:
        """
        Send the text messages return and the number of messages sent.
        """
        if not self.recipients:
            # Don't brother creating the network connection if there's nobody
            # to send the text message to
            return 0
        connection: Type['BaseSmsBackend'] = self.get_connection(fail_silently)
        count = connection.send_messages([self])  # type: ignore

        post_send.send(sender=self.__class__, instance=self)

        return count



"""
Tools for sending text messages.
"""
from typing import List, Optional, Type, Union

from django.conf import settings  # type: ignore
from django.utils.module_loading import import_string  # type: ignore

from sms.backends.base import BaseSmsBackend
from sms.message import Message

__all__ = [
    'Message', 'get_connection', 'send_sms'
]


def get_connection(
    backend: Optional[str] = None,
    fail_silently: bool = False,
    **kwargs
) -> Type[BaseSmsBackend]:
    """Load a SMS backend and return an instance of it.

    If backend is None (default), use settings.SMS_BACKEND.

    Both fail_silently and other keyword arguments are used in the constructor
    of the backend.
    """
    klass = import_string(backend or settings.SMS_BACKEND)
    return klass(fail_silently=fail_silently, **kwargs)


def send_sms(
    body: str = '',
    originator: Optional[str] = None,
    recipients: Union[Optional[str], Optional[List[str]]] = None,
    fail_silently: bool = False,
    connection: Optional[Type['BaseSmsBackend']] = None
) -> int:
    """
    Easy wrapper for sending a single message to a recipient list.

    Allow to to be a string, to remain compatibility with older
    django-sms<=0.0.4.

    If originator is None, use DEFAULT_FROM_SMS setting.

    Note: The API for this method is frozen. New code wanting to extend the
    functionality should the the Message class directly.
    """
    if isinstance(recipients, str):
        recipients = [recipients]
    msg = Message(body, originator, recipients, connection=connection)
    return msg.send(fail_silently=fail_silently)



"""
SMS backend for sending text messages using MessageBird.
"""
from typing import List, Optional

from django.conf import settings  # type: ignore
from django.core.exceptions import ImproperlyConfigured  # type: ignore

from sms.backends.base import BaseSmsBackend
from sms.message import Message

try:
    import messagebird  # type: ignore
    HAS_MESSAGEBIRD = True
except ImportError:
    HAS_MESSAGEBIRD = False


class SmsBackend(BaseSmsBackend):
    def __init__(self, fail_silently: bool = False, **kwargs) -> None:
        super().__init__(fail_silently=fail_silently, **kwargs)

        if not HAS_MESSAGEBIRD and not self.fail_silently:
            raise ImproperlyConfigured(
                "You're using the SMS backend "
                "'sms.backends.messagebird.SmsBackend' without having "
                "'messagebird' installed. Install 'messagebird' or use "
                "another SMS backend."
            )

        access_key: Optional[str] = getattr(settings, 'MESSAGEBIRD_ACCESS_KEY')
        if not access_key and not self.fail_silently:
            raise ImproperlyConfigured(
                "You're using the SMS backend "
                "'sms.backends.messagebird.SmsBackend' without having the "
                "setting 'MESSAGEBIRD_ACCESS_KEY' set."
            )

        self.client = None
        if HAS_MESSAGEBIRD:
            self.client = messagebird.Client(access_key)

    def send_messages(self, messages: List[Message]) -> int:
        if not self.client:
            return 0

        msg_count: int = 0
        for message in messages:
            try:
                self.client.message_create(
                    message.originator,
                    message.recipients,
                    message.body
                )
            except Exception as exc:
                if not self.fail_silently:
                    raise exc
            msg_count += 1
        return msg_count



"""A custom backend for testing."""

from typing import List

from sms.backends.base import BaseSmsBackend
from sms.message import Message


class SmsBackend(BaseSmsBackend):

    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self.test_outbox: List[Message] = []

    def send_messages(self, messages: List[Message]) -> int:
        # Messages are stored in an instance variable for testing
        self.test_outbox.extend(messages)
        return len(messages)



import re

from sms.message import Message


header_RE = re.compile('^(?:from|to): .*')
header_from_RE = re.compile('^from: (.*)$', re.MULTILINE)
header_to_RE = re.compile('^to: (.*)$', re.MULTILINE)


def message_from_binary_file(fp) -> Message:
    """Parse a binary file into a Message object model."""
    return message_from_bytes(fp.read())


def message_from_bytes(s: bytes) -> Message:
    """Parse a bytes string into a Message object model."""
    text = s.decode('ASCII', errors='surrogateescape')

    body = ''
    for line in text.splitlines():
        if not header_RE.match(line):
            if body is not None:
                body += '\n' + line
            else:
                body = line

    from_result = header_from_RE.search(text)
    if from_result:
        from_phone = from_result.group(1)
    else:
        raise ValueError

    to_result = header_to_RE.search(text)
    if to_result:
        to = [to_result.group(1)]
    else:
        raise ValueError

    return Message(body, from_phone, to)



"""Base SMS backend class."""
from typing import Optional, Type, List
from types import TracebackType

from sms.message import Message


class BaseSmsBackend:
    """
    Base class for sms backend implementations.

    Subclasses must at least overwrite send_messages().

    open() and close() can be called indirectly by using a backend object as a
    context manager:

        with backend as connection:
            # do something with connection
            pass
    """
    def __init__(self, fail_silently: bool = False, **kwargs) -> None:
        self.fail_silently = fail_silently

    def open(self) -> bool:
        """
        Open a network connection.

        This method can be overwritten by backend implementations to open a
        network connection.

        It's up to the backend implementation to track the status of a network
        connection if it's needed by the backend.

        This method can be called by application to force a single network
        connection to be used when sending text messages.

        The default implementation does nothing.
        """
        return True

    def close(self) -> None:
        """Close a network connection."""
        pass

    def __enter__(self) -> 'BaseSmsBackend':
        try:
            self.open()
        except Exception:
            self.close()
            raise
        return self

    def __exit__(
        self,
        exc_type: Optional[Type[BaseException]],
        exc_value: Optional[BaseException],
        traceback: Optional[TracebackType]
    ) -> None:
        self.close()

    def send_messages(
        self,
        messages: List[Message]
    ) -> int:
        """
        Send one or more Message objects and return the number of text messages
        sent.
        """
        raise NotImplementedError(
            'subclasses of BaseSmsBacked must override send_messages() method'
        )



#!/usr/bin/python
# -*- coding: utf-8 -*-

from setuptools import find_packages, setup


def long_description() -> str:
    """
    Returns the long description containing both the README.md and CHANGELOG.md
    files.
    """
    with open('README.md') as readme_file:
        readme = readme_file.read()

    with open('CHANGELOG.md') as changelog_file:
        changelog = changelog_file.read()

    return readme + changelog


setup(
    name='django-sms',
    version='0.7.0',
    url="https://github.com/django-enterprise/django-sms",
    description='A Django app for sending SMS with interchangeable backends.',
    long_description=long_description(),
    long_description_content_type='text/markdown',
    author='Roald Nefs',
    author_email='info@roaldnefs.com',
    license='BSD-3-Clause',
    platforms='any',
    zip_safe=False,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 3.2',
        'Framework :: Django :: 4.1',
        'Framework :: Django :: 4.2',
        'Framework :: Django :: 5.0',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
    ],
    packages=find_packages(exclude=['tests', 'tests.*']),
    include_package_data=True,
    test_suite='tests.runtests.main',
    install_requires=['Django>=3.2'],
    extras_require={
        'messagebird': ['messagebird'],
        'twilio': ['twilio'],
    }
)



"""
SMS backend that writes messages to a file.
"""
import datetime
import os
from typing import Optional

from django.conf import settings  # type: ignore
from django.core.exceptions import ImproperlyConfigured  # type: ignore

from sms.backends.console import SmsBackend as BaseSmsBackend
from sms.message import Message


class SmsBackend(BaseSmsBackend):
    file_path: str

    def __init__(
        self,
        *args,
        file_path: Optional[str] = None,
        **kwargs
    ) -> None:
        self._fname: Optional[str] = None
        if not file_path:
            file_path = getattr(settings, 'SMS_FILE_PATH', None)
        if not file_path:
            raise ImproperlyConfigured((
                'Missin path for saving text messages'
            ))
        self.file_path = file_path

        self.file_path: str = os.path.abspath(self.file_path)
        try:
            os.makedirs(self.file_path, exist_ok=True)
        except FileExistsError:
            raise ImproperlyConfigured((
                'Path for saving text messages exists, but is not a '
                f'directory: {self.file_path}'
            ))
        except OSError as exc:
            raise ImproperlyConfigured((
                'Could not create directory for saving text messages: '
                f'{self.file_path} ({exc})'
            ))
        # Make sure that self.file_path is writable.
        if not os.access(self.file_path, os.W_OK):
            raise ImproperlyConfigured(
                f'Could not write to directory: {self.file_path}'
            )
        # Finally, call super().
        # Since we're using the console-based backend as a base,
        # force the stream to be None, so we don't default to stdout
        kwargs['stream'] = None
        super().__init__(*args, **kwargs)

    def write_message(self, message: Message) -> int:
        msg_count = 0
        for recipient in message.recipients:
            msg_data = (
                f"from: {message.originator}\n"
                f"to: {recipient}\n"
                f"{message.body}"
            )
            self.stream.write(f'{msg_data}\n'.encode())
            self.stream.write(b'-' * 79)
            self.stream.write(b'\n')
            msg_count += 1
        return msg_count

    def _get_filename(self) -> str:
        """Return a unique file name."""
        if self._fname is None:
            timestamp = datetime.datetime.now().strftime("%Y%m%d-%H%M%S")
            fname = "%s-%s.log" % (timestamp, abs(id(self)))
            self._fname = os.path.join(self.file_path, fname)
        return self._fname

    def open(self) -> bool:
        if self.stream is None:
            self.stream = open(self._get_filename(), 'ab')
            return True
        return False

    def close(self) -> None:
        try:
            if self.stream is not None:
                self.stream.close()
        finally:
            self.stream = None
